//+------------------------------------------------------------------+
//|                     AmazingEA.mq4 Version 2.2.0 By Alan Prothero |
//|               See "AmazingEA Change Log.txt" for Update History  |
//+------------------------------------------------------------------+
#property copyright "Alan Prothero"

extern int PointsAway=50; // Points Away, number of points above and below the current price where the orders will be placed
extern int TP=1000; // Take Profit, exact amount of points from open price (set to 0 to disable)
extern int SL=50; // Stop Loss, hard stop from open price (set to 0 to disable)
extern int NYear=0;  // News Year, broker time not PC time (set to 0 to enable EA every day)
extern int NMonth=0;  // News Month, broker time not PC time (set to 0 to enable EA every day)
extern int NDay=0;  // News Day, broker time not PC time (set to 0 to enable EA every day)
extern int NHour=23; // News Hour, broker time not PC time
extern int NMin=59;  // News Minute, broker time not PC time
extern int CTCBN=1; // Numbers of Candles To Check Before News, for determining highs & lows (when it is 1 it checks the current candle) (set to 0 to disable, 1 for default)
extern int SecBPO=10; // Seconds Before Pending Orders, number of seconds before news when pending orders will be placed
extern int SecBMO=0; // Seconds Before Modifying Orders, number of seconds before news when orders will stop being modified
extern int STWAN=5; // Seconds To Wait After News, number of seconds after news to wait before deleting pending orders
extern bool OCO=true; // Order Cancel Other, will cancel the other pending order if one of them is activated but only after news time
extern int BEPoints=50; // Break Even Points, number of points in profit before the stop loss is moved to Break Even + BEOffset (set to 0 to disable)
extern int BEOffset=0; // Break Even Offset, number of points to move beyond Break Even (allows to cover Broker Commissions etc.) (set to 0 to disable)
extern int TrailPoints=0; // Trailing Stop, number of points to a trailing stop (set to 0 to disable)
extern int TrailOffset=0; // Trailing Stop Offset, number of points profit required before trailing stop (set to 0 to disable)
extern bool MM=false; // Money Management, if true uses Risk Percentage, if false uses Lot Size
extern int RiskPercent=1; // Risk Percentage, percent amount of account to risk when Money Management set to true, overrides Lot Size
extern double Lots=0.1; // Lot Size, number of lots to trade when Money Management set to false
extern bool CloseTrades=false; // Changing the chart timeframe or removing EA closes any trades that are opened by this instance of the EA (set to false to disable)
extern int MaxSpread=100; // Maximum Spread, Cancels Pending Orders if Spread exceeds this setting (set to 0 to disable)
extern bool AddSpreadToSL=true; // Adds the Spread to the Stop Loss, if true you can set lower SL
extern bool SlipCheck=false; // Checks for Slippage and Stop Loss Reset, set to false to replicate 'old' EA
extern int MaxSlippage=200; // Maximum Slippage, if the slippage (open price vs. requested price) exceeds this setting then set the minimum stop loss on the trade (set to 0 to disable).
extern string TradeLog = "AmazingEA"; // Log is created in experts/files sub-directory with this name followed by the CurrencySymbol followed by .log as an extension
double h,l,ho,lo,hso,lso,htp,ltp,sp;
int Magic;
string filename;
string TradeComment;

double LotsOptimized()
{
	double lot=Lots;
	//---- select lot size
	if (MM) lot=NormalizeDouble(MathFloor(AccountFreeMargin()*RiskPercent/100)/100,2);
	return(lot);
} 

string AddLeadingZero(int number, int digits)
{
	// add leading zeros that the resulting string has 'digits' length.
	string result;
	result = DoubleToStr(number, 0);
	while(StringLen(result)<digits) result = "0"+result;
	return(result);
}

int CheckOrdersCondition()
{
	int result=0;
	for (int i=0;i<OrdersTotal();i++) {
		OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
		if ((OrderType()==OP_BUY) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
			result=result+1000; 
		}
		if ((OrderType()==OP_SELL) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
			result=result+100; 
		}
		if ((OrderType()==OP_BUYSTOP) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
			result=result+10;
		}
		if ((OrderType()==OP_SELLSTOP) && (OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
			result=result+1; 
		}
	}
	return(result); // 0 means we have no trades
}

// OrdersCondition Result Pattern
//    1    1    1    1
//    b    s    bs   ss
//  


void OpenBuyStop()
{
	int ticket,err,tries;
	tries = 0;
	if (!GlobalVariableCheck("InTrade")) {
		while (tries < 3)
		{
			GlobalVariableSet("InTrade", CurTime());  // set lock indicator
			ticket = OrderSend(Symbol(),OP_BUYSTOP,LotsOptimized(),ho,1,hso,htp,TradeComment,Magic,0,Green);
			Write("OpenBuyStop, OrderSend Executed, ticket="+ticket);
			GlobalVariableDel("InTrade");   // clear lock indicator
			if(ticket<=0) {
				Write("Error Occurred : "+ErrorDescription(GetLastError())+" BuyStop @ "+ho+" SL @ "+hso+" TP @"+htp);
				tries++;
			} else tries = 3;
		}
	}
}

void OpenSellStop()
{
	int ticket,err,tries;
	tries = 0;
	if (!GlobalVariableCheck("InTrade")) {
		while (tries < 3)
		{
			GlobalVariableSet("InTrade", CurTime());  // set lock indicator
			ticket = OrderSend(Symbol(),OP_SELLSTOP,LotsOptimized(),lo,1,lso,ltp,TradeComment,Magic,0,Red);
			Write("OpenSellStop, OrderSend Executed, ticket="+ticket);
			GlobalVariableDel("InTrade");   // clear lock indicator
			if(ticket<=0) {
				Write("Error Occurred : "+ErrorDescription(GetLastError())+" SellStop @ "+lo+" SL @ "+lso+" TP @"+ltp);
				tries++;
			} else tries = 3;
		}
	}
}

void DoModify()
{
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_BUYSTOP)) {
			if ((OrderOpenPrice()>ho) || (OrderOpenPrice()<ho)) {
				Write("Buy Stop was @ "+DoubleToStr(OrderOpenPrice(),5)+", changed to "+DoubleToStr(ho,5));
				OrderModify(OrderTicket(),ho,hso,htp,0,Green);
			}
		}
		if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_SELLSTOP)) {
			if ((OrderOpenPrice()>lo) || (OrderOpenPrice()<lo)) {
				Write("Sell Stop was @ "+DoubleToStr(OrderOpenPrice(),5)+", changed to "+DoubleToStr(lo,5));
				OrderModify(OrderTicket(),lo,lso,ltp,0,Red);
			}
		}
	}
}

void DoSlip()
{
	int MinStopLevel=MarketInfo(Symbol(),MODE_STOPLEVEL); // Min. distance for Stops
	int slippage;
	double osl,sl,be,otp,tp;
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic)) { // only look if mygrid and symbol...
			otp=OrderTakeProfit();
			osl=OrderStopLoss();
			if (OrderType() == OP_BUY) {
				be=OrderOpenPrice() + ((BEPoints + BEOffset) * Point);
				tp=OrderOpenPrice() + (TP * Point);
				slippage=NormalizeDouble((tp - otp)/Point,0); // slippage is how far Take Profit is out by
				if (MaxSlippage!=0 && slippage > MaxSlippage) // if slippage exceeds maxslippage
				{
					if (AddSpreadToSL) { // set MinStop
						sl=OrderOpenPrice() - sp - (MinStopLevel * Point);
					}
					else {
						sl=OrderOpenPrice() - (MinStopLevel * Point);
					}
					Write("Slippage of Buy Order was "+slippage+", exceeded MaxSlippage of "+MaxSlippage+", setting MinStop to "+DoubleToStr(sl,5));
				}
				else // if slippage does not exceed maxslippage
				{
					if (slippage > 0) { //output slippage to log
						Write("Slippage of Buy Order was "+slippage);
					}
					if (AddSpreadToSL) { // set normal stop
						sl=OrderOpenPrice() - sp - (SL * Point);
					}
					else {
						sl=OrderOpenPrice() - (SL * Point);
					}
				}
				if (Bid > tp)
				// if bid higher than buy take profit level, close
				{
					Write("Take Profit of Buy Order Hit at "+DoubleToStr(tp,5)+", Closing at "+DoubleToStr(Bid,5));
					OrderClose(OrderTicket(),OrderLots(),Bid,0,Green);
				}
				if (BEPoints==0 || Bid < be) {
					if (osl < sl) 
					// is bid lower than break-even (open + BE) and 
					// is stop loss lower than where it should be
					{
						if (Bid < sl)
						// if bid lower than buy stop level, close 
						{
							Write("Stop Loss of Buy Order Hit at "+DoubleToStr(sl,5)+", Closing at "+DoubleToStr(Bid,5));
							OrderClose(OrderTicket(),OrderLots(),Bid,0,Green);
						}
						if (Bid > sl) 
						// if bid higher than sl, reset stop loss and take profit
						{
							Write("Stop Loss of Buy Order Reset to "+DoubleToStr(sl,5)+", Take Profit of Buy Order Reset to "+DoubleToStr(tp,5)+" at "+DoubleToStr(Bid,5));
							OrderModify(OrderTicket(), OrderOpenPrice(), sl, tp, Green);
						}
					}
				}
			}
			if (OrderType() == OP_SELL) {
				be=OrderOpenPrice() - ((BEPoints + BEOffset) * Point);
				tp=OrderOpenPrice() - (TP * Point);
				slippage=NormalizeDouble((otp - tp)/Point,0); // slippage is how far Take Profit is out by
				if (MaxSlippage!=0 && slippage > MaxSlippage) // if slippage exceeds maxslippage
				{
					if (AddSpreadToSL) { // set MinStop
						sl=OrderOpenPrice() + sp + (MinStopLevel * Point);
					}
					else {
						sl=OrderOpenPrice() + (MinStopLevel * Point);
					}
					Write("Slippage of Sell Order was "+slippage+", exceeded MaxSlippage of "+MaxSlippage+", setting MinStop to "+DoubleToStr(sl,5));
				}
				else // if slippage does not exceed maxslippage
				{
					if (slippage > 0) { //output slippage to log
						Write("Slippage of Sell Order was "+slippage);
					}
					if (AddSpreadToSL) { // set normal stop
						sl=OrderOpenPrice() + sp + (SL * Point);
					}
					else {
						sl=OrderOpenPrice() + (SL * Point);
					}
				}
				if (Ask < tp) 
				// if ask lower than sell take profit level, close
				{
					Write("Take Profit of Sell Order Hit at "+DoubleToStr(tp,5)+", Closing at "+DoubleToStr(Ask,5));
					OrderClose(OrderTicket(),OrderLots(),Ask,0,Red);
				}
				if (BEPoints==0 || Ask > be) {
					if (osl > sl)
					// is ask higher than break-even (open - BE) and 
					// is stop loss higher than where it should be
					{
						if (Ask > sl) 
						// if ask higher than sell stop level, close
						{
							Write("Stop Loss of Sell Order Hit at "+DoubleToStr(sl,5)+", Closing at "+DoubleToStr(Ask,5));
							OrderClose(OrderTicket(),OrderLots(),Ask,0,Red);
						}
						if (Ask < sl) 
						// if ask lower than sl reset stop loss and take profit
						{ 
							Write("Stop Loss of Sell Order Reset to "+DoubleToStr(sl,5)+", Take Profit of Sell Order Reset to "+DoubleToStr(tp,5)+" at "+DoubleToStr(Bid,5));
							OrderModify(OrderTicket(), OrderOpenPrice(), sl, tp, Red);
						}
					}
				}
			}
		}
	}
}

void DoSL()
{
	double osl,be,sl;
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic)) { // only look if mygrid and symbol...
			osl=OrderStopLoss();
			if (OrderType() == OP_BUY) {
				be=OrderOpenPrice() + ((BEPoints + BEOffset) * Point);
				if (AddSpreadToSL)
				{
					sl=OrderOpenPrice() - sp - (SL * Point);
				}
				else
				{
					sl=OrderOpenPrice() - (SL * Point);
				}
				if (BEPoints==0 || Bid < be){
					if (osl < sl) 
					// is bid lower than break-even (open + BE) and 
					// is stop loss lower than where it should be
					{
						if (Bid < sl)
						// if bid lower than buy stop level, close 
						{
							Write("Stop Loss of Buy Order Hit at "+DoubleToStr(sl,5)+", Closing at "+DoubleToStr(Bid,5));
							OrderClose(OrderTicket(),OrderLots(),Bid,0,Green);
						}
						if (Bid > sl) 
						// if bid higher than sl, reset stop loss set to open minus stop loss
						{
							Write("Stop Loss of Buy Order Reset to "+DoubleToStr(sl,5)+", at "+DoubleToStr(Bid,5));
							OrderModify(OrderTicket(), OrderOpenPrice(), sl, OrderTakeProfit(), Green);
						}
					}
				}
			}
			if (OrderType() == OP_SELL) {
				be=OrderOpenPrice() - ((BEPoints + BEOffset) * Point);
				if (AddSpreadToSL)
				{
					sl=OrderOpenPrice() + sp + (SL * Point);
				}
				else
				{
					sl=OrderOpenPrice() + (SL * Point);
				}
				if (BEPoints==0 || Ask > be) {
					if (osl > sl)
					// is ask higher than break-even (open - BE) and 
					// is stop loss higher than where it should be
					{
						if (Ask > sl) 
						// if ask higher than sell stop level, close
						{
							Write("Stop Loss of Sell Order Hit at "+DoubleToStr(sl,5)+", Closing at "+DoubleToStr(Ask,5));
							OrderClose(OrderTicket(),OrderLots(),Ask,0,Red);
						}
						if (Ask < sl) 
						// if ask lower than sl reset stop loss set to open plus stop loss
						{ 
							Write("Stop Loss of Sell Order Reset to "+DoubleToStr(sl,5)+", at "+DoubleToStr(Ask,5));
							OrderModify(OrderTicket(), OrderOpenPrice(), sl, OrderTakeProfit(), Red);
						}
					}
				}
			}
		}
	}
}

void DoTP()
{
	double otp,tp;
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic)) { // only look if mygrid and symbol...
			otp=OrderTakeProfit();
			if (OrderType() == OP_BUY) {
				tp=OrderOpenPrice() + (TP * Point);
				if (otp < tp || otp > tp) 
				// is take profit lower than where it should be
				{
					if (Bid > tp)
					// if bid higher than buy take profit level, close
					{
						Write("Take Profit of Buy Order Hit at "+DoubleToStr(tp,5)+", Closing at "+DoubleToStr(Bid,5));
						OrderClose(OrderTicket(),OrderLots(),Bid,0,Green);
					}
					if (Bid < tp) 
					// if bid lower than tp, reset take profit to open plus take profit setting
					{
						Write("Take Profit of Buy Order Reset to "+DoubleToStr(tp,5)+", at "+DoubleToStr(Bid,5));
						OrderModify(OrderTicket(), OrderOpenPrice(), OrderStopLoss(), tp, Green);
					}
				}
			}
			if (OrderType() == OP_SELL) {
				tp=OrderOpenPrice() - (TP * Point);
				if (otp < tp || otp > tp)
				// is stop loss higher than where it should be
				{
					if (Ask < tp) 
					// if ask lower than sell take profit level, close
					{
						Write("Take Profit of Sell Order Hit at "+DoubleToStr(tp,5)+", Closing at "+DoubleToStr(Ask,5));
						OrderClose(OrderTicket(),OrderLots(),Ask,0,Red);
					}
					if (Ask > tp) 
					// if ask lower than sl reset stop loss set to open plus stop loss
					{ 
						Write("Take Profit of Sell Order Reset to "+DoubleToStr(tp,5)+", at "+DoubleToStr(Ask,5));
						OrderModify(OrderTicket(), OrderOpenPrice(), OrderStopLoss(), tp, Red);
					}
				}
			}
		}
	}
}

void DoBE()
{
	double osl,be,of;
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic)) { // only look if mygrid and symbol...
			osl=OrderStopLoss();
			if (OrderType() == OP_BUY) {
				be=OrderOpenPrice() + ((BEPoints + BEOffset) * Point);
				of=OrderOpenPrice() + (BEOffset * Point);
				if (Bid > be && (osl < of || osl==0))
				// is bid higher than break-even (open + BE) and 
				// is stop loss lower than open plus BE offset
				{
					Write("Break Even of Buy Order set to "+DoubleToStr(of,5)+" at "+DoubleToStr(Bid,5));
					OrderModify(OrderTicket(), OrderOpenPrice(), of, OrderTakeProfit(), Green);
				}
			}
			if (OrderType() == OP_SELL) {
				be=OrderOpenPrice() - ((BEPoints + BEOffset) * Point);
				of=OrderOpenPrice() - (BEOffset * Point);
				if (Ask < be && (osl > of || osl==0))
				// is ask lower than break-even (open - BE) and 
				// is stop loss higher than open minus BE offset
				{
					Write("Break Even of Sell Order set to "+DoubleToStr(of,5)+" at "+DoubleToStr(Ask,5));
					OrderModify(OrderTicket(), OrderOpenPrice(), of, OrderTakeProfit(), Red);
				}
			}
		}
	}
}

void DoTrail()
{
	double osl,to,tl;
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic)) { // only look if mygrid and symbol...
			osl=OrderStopLoss();
			if (OrderType() == OP_BUY) {
				to=OrderOpenPrice() + ((TrailPoints + TrailOffset) * Point);
				tl=Bid - (TrailPoints * Point);
				if (Bid > to && (osl < tl || osl==0))
				// is bid higher than open plus trail and offset setting and
				// is stop loss less than bid minus the trail setting
				{
					Write("Trailing Stop of Buy Order set to "+DoubleToStr(tl,5)+" at "+DoubleToStr(Bid,5));
					OrderModify(OrderTicket(), OrderOpenPrice(), tl, OrderTakeProfit(), Green);
				}
			}
			if (OrderType() == OP_SELL) {
				to=OrderOpenPrice() - ((TrailPoints + TrailOffset)* Point);
				tl=Ask + (TrailPoints * Point);
				if (Ask < to && (osl > tl || osl==0))
				// is ask lower than open minus trail and offset setting and
				// is stop loss higher than ask plus the trail setting
				{
					Write("Trailing Stop of Sell Order set to "+DoubleToStr(tl,5)+" at "+DoubleToStr(Ask,5));
					OrderModify(OrderTicket(), OrderOpenPrice(), tl, OrderTakeProfit(), Red);
				}
			}
		}
	}
}

void CloseAll()
{
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic)) { // only look if mygrid and symbol...
			if (OrderType() == OP_BUY) 
			{
				Write("Closing Buy Order at "+DoubleToStr(Bid,5));
				OrderClose(OrderTicket(),OrderLots(),Bid,0,Green);
			}
			if (OrderType() == OP_SELL) 
			{
				Write("Closing Sell Order at "+DoubleToStr(Ask,5));
				OrderClose(OrderTicket(),OrderLots(),Ask,0,Red);
			}
		}
	}
}

void DeleteBuyStop()
{
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_BUYSTOP)) {
			OrderDelete(OrderTicket());
			Write("DeleteBuyStop, OrderDelete Executed");
		}
		
	}
}

void DeleteSellStop()
{
	for (int i = 0; i < OrdersTotal(); i++) {
		OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
		if (OrderSymbol()==Symbol() && (OrderMagicNumber() == Magic) && (OrderType()==OP_SELLSTOP)) {
			OrderDelete(OrderTicket());
			Write("DeleteSellStop, OrderDelete Executed");
		}
		
	}
}

void OrdersDeleteAll()
{
	DeleteBuyStop();
	DeleteSellStop();
}

int Write(string str)
{
	int handle;
	handle = FileOpen(filename,FILE_READ|FILE_WRITE|FILE_CSV,"/t");
	FileSeek(handle, 0, SEEK_END);      
	FileWrite(handle, TimeToStr(CurTime(),TIME_DATE|TIME_SECONDS) + " " + str);
	FileClose(handle);
}

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
	ObjectsDeleteAll();
	return(0);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
	Comment("");
	Write("Amazing EA shut down");
	OrdersDeleteAll();
	if (CloseTrades) CloseAll(); // if user has enabled close trades on change of timeframe
	return(0);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{

	Magic=NYear+NMonth+NDay+NHour+NMin;

	int i;
	int secofday,secofnews;
	int OrdersCondition=CheckOrdersCondition();
	int MinStopLevel=MarketInfo(Symbol(),MODE_STOPLEVEL); // Min. distance for Stops
	filename=TradeLog + "-" + Symbol() + ".log";

	if (PointsAway < MinStopLevel ) { 
		PointsAway = MinStopLevel ; 
	}

	if (OrdersCondition > 11) { // we have open trades, amending stops
		if (SlipCheck && SL!=0 && TP!=0) DoSlip(); // SL and TP both set, work out slippage
		if (SlipCheck && SL!=0 && TP==0) DoSL(); // SL set so can reset SL, TP not set so can't workout slippage
		if (SlipCheck && SL==0 && TP!=0) DoTP(); // TP set but no SL, no point working out slippage
		if (BEPoints!=0) DoBE(); // perform break even processing
		if (TrailPoints!=0) DoTrail(); // perform trailing stop processing
	}

	secofday=Hour()*3600+Minute()*60+Seconds();
	secofnews=NHour*3600+NMin*60;

	if (CTCBN==0) 
	{
		h=Bid;
		l=Bid;
	}
	else
	{
		h=iHigh(NULL,0,0);
		l=iLow(NULL,0,0);
		for (i=1;i<=CTCBN;i++) if (iHigh(NULL,0,i-1)>h) h=iHigh(NULL,0,i-1);
		for (i=1;i<=CTCBN;i++) if (iLow(NULL,0,i-1)<l) l=iLow(NULL,0,i-1);
	}

	sp=Ask-Bid;

	int spread=NormalizeDouble(sp/Point,0);

	ho=h+sp+(PointsAway*Point); 
	lo=l-(PointsAway*Point);

	if (SL==0)
	{
		hso = 0;
		lso = 0;
	}
	else if (AddSpreadToSL)
	{
		hso=ho-sp-(SL*Point); //Bid+(PointsAway*Point)-(SL*Point); //hso=Ask+(PipsAway-SL)*Point; //hso=h+sp;
		lso=lo+sp+(SL*Point); //Ask-(PointsAway*Point)+(SL*Point); //lso=Bid-(PipsAway-SL)*Point; //lso=l;
	}
	else
	{
		hso=ho-(SL*Point); //Ask+(PointsAway*Point)-(SL*Point)
		lso=lo+(SL*Point); //Bid-(PointsAway*Point)+(SL*Point)
	}
	
	if (TP==0)
	{
		htp = 0;
		ltp = 0;
	}
	else
	{ 
		htp=ho+(TP*Point);
		ltp=lo-(TP*Point);
	}

	string title="\nAmazing Forex System Expert Advisor v 2.2.0 By Alan Prothero\n\n";
	string brokertime=TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS); // BrokerTime is shown in the EA Comment.
	string newstime=StringConcatenate(AddLeadingZero(NYear,4),".",AddLeadingZero(NMonth,2),".",AddLeadingZero(NDay,2)," ",TimeToStr(secofnews,TIME_SECONDS));
	string timetitle=StringConcatenate("Broker Time : ", brokertime, "\nNews Time  : ", newstime, "\n\n");
	string DisplayOCO="False";
	if (OCO) DisplayOCO="True";
	string DisplayMM="False";
	if (MM) DisplayMM="True";
	string DisplayCT="False";
	if (CloseTrades) DisplayCT="True";
	string DisplayAddSpreadToSL="False";
	if (AddSpreadToSL) DisplayAddSpreadToSL="True";
	string DisplaySlipCheck="False";
	if (SlipCheck) DisplaySlipCheck="True";
	
	// TradeComment gets added in the Comment field of trades. Max 32 chars.
	// Pipe character denotes different section, helping to logically group parameters.
	TradeComment=StringConcatenate("P", PointsAway, "S", SL, "T", TP, "C", CTCBN, "|P", SecBPO, "M", SecBMO, "W", STWAN, "O", OCO);

	if (MaxSpread!=0)
	{
		if (spread > MaxSpread) {
			Write("MaxSpread Exceeded, MaxSpread: "+MaxSpread+" Spread : "+spread);
			OrdersDeleteAll();
			Comment(title, timetitle, "MaxSpread : ",MaxSpread," | Spread : ",spread,"\n\nExpert is disabled because Spread exceeds MaxSpread Setting");
			// Despite the comment above, the expert is not really disabled, it just exits without trading.
			// The return statement below is very important as it ensures the EA exits without opening trades if the Spread is too high. 
			return (0);
		}
	}

	if (( NYear==Year() && NMonth==Month() && NDay==Day() ) || ( NYear==0 && NMonth==0 && NDay==0 ))
	{
		Comment(title, timetitle, "High @ ",DoubleToStr(h+sp,5)," BuyOrder @ ",DoubleToStr(ho,5)," StopLoss @ ",DoubleToStr(hso,5)," TakeProfit @ ",DoubleToStr(htp,5),"\nLow  @ ",DoubleToStr(l,5)," SellOrder @ ",DoubleToStr(lo,5)," StopLoss @ ",DoubleToStr(lso,5)," TakeProfit @ ",DoubleToStr(ltp,5),"\n\nPointsAway : ",PointsAway," | BEOffset : ",BEOffset," | BEPoints : ",BEPoints," | TrailOffset : ", TrailOffset," | TrailPoints : ", TrailPoints,"\n\nCTCBN : ",CTCBN," | SecBPO : ",SecBPO," | SecBMO : ",SecBMO," | STWAN : ",STWAN," | OCO : ",DisplayOCO,"\n\nMoney Management : ",DisplayMM," | RiskPercent: ",RiskPercent," | Lots : ",LotsOptimized(),"\n\nCloseTrades : ",DisplayCT," | AddSpreadToSL : ",DisplayAddSpreadToSL," | SlipCheck : ",DisplaySlipCheck,"\n\nMaxSpread : ",MaxSpread," | Spread : ",spread," | MaxSlippage : ",MaxSlippage);
	}
	else
	{
		Comment(title, timetitle, "Expert is disabled because it is not day of expected news");
		// Despite the comment above, the expert is not really disabled, it just exits without trading.
		// The return statement below is very important as it ensures the EA exits without opening trades on non-news days. 
		return(0);
	} 

	// OrdersCondition Result Pattern
	//    1    1    1    1
	//    b    s    bs   ss
	//  

	if ((secofday<secofnews) && (secofday>(secofnews-SecBPO))) { // if before news but after news minus BPO
		
		if (OrdersCondition==0) { // if we have no orders
			Write("Opening BuyStop @ "+DoubleToStr(ho,5)+" & SellStop @ "+DoubleToStr(lo,5)+", OrdersCondition="+OrdersCondition);
			OpenBuyStop();
			OpenSellStop();
		}
		if (OrdersCondition==1) { // if we only have a sell stop
			Write("Opening BuyStop @ "+DoubleToStr(ho,5)+", OrdersCondition="+OrdersCondition);
			OpenBuyStop();
		}
		if (OrdersCondition==10) { // if we only have a buy stop
			Write("Opening SellStop @ "+DoubleToStr(lo,5)+", OrdersCondition="+OrdersCondition);
			OpenSellStop();
		}
	}

	if ((secofday<secofnews) && (secofday>(secofnews-SecBPO)) && (secofday<(secofnews-SecBMO))) { // if before news but after news minus BPO and before news minus BMO
		Write("Modifying Orders, OrdersCondition="+OrdersCondition);
		DoModify();
	}
	

	if ((secofday>secofnews) && (secofday<(secofnews+STWAN)) && OCO) { // if after news and within wait time and we are using one cancels other

		if (OrdersCondition==1001) { // if we have a buy and a sell stop
			Write("Deleting SellStop because BuyStop Hit, OrdersCondition="+OrdersCondition);
			DeleteSellStop();
		}
		if (OrdersCondition==110) { // if we have a sell and a buy stop
			Write("Deleting BuyStop because SellStop Hit, OrdersCondition="+OrdersCondition);
			DeleteBuyStop();
		}
	}

	if ((secofday>secofnews) && (secofday>(secofnews+STWAN))) { // if after news and after wait time 
		if (OrdersCondition==11) { // if we have a buy stop and a sell stop
			Write("Deleting BuyStop and SellStop because STWAN expired, OrdersCondition="+OrdersCondition);
			DeleteSellStop();
			DeleteBuyStop();
		}
		if ((OrdersCondition==1) || (OrdersCondition==1001)) { // if we have a sell stop or a buy and a sell stop
			Write("Deleting SellStop because STWAN expired, OrdersCondition="+OrdersCondition);
			DeleteSellStop();
		}
		if ((OrdersCondition==10) || (OrdersCondition==110)) { // if we have a buy stop or a sell and a buy stop
			Write("Deleting BuyStop because STWAN expired, OrdersCondition="+OrdersCondition);
			DeleteBuyStop();
		}
	}

	//----
	return(0);
}

//+------------------------------------------------------------------+
//| return error description                                         |
//+------------------------------------------------------------------+
string ErrorDescription(int error_code)
{
	string error_string;
	//----
	switch(error_code)
	{
		//---- codes returned from trade server
	case 0:
	case 1:   error_string="no error";                                                  break;
	case 2:   error_string="common error";                                              break;
	case 3:   error_string="invalid trade parameters";                                  break;
	case 4:   error_string="trade server is busy";                                      break;
	case 5:   error_string="old version of the client terminal";                        break;
	case 6:   error_string="no connection with trade server";                           break;
	case 7:   error_string="not enough rights";                                         break;
	case 8:   error_string="too frequent requests";                                     break;
	case 9:   error_string="malfunctional trade operation";                             break;
	case 64:  error_string="account disabled";                                          break;
	case 65:  error_string="invalid account";                                           break;
	case 128: error_string="trade timeout";                                             break;
	case 129: error_string="invalid price";                                             break;
	case 130: error_string="invalid stops";                                             break;
	case 131: error_string="invalid trade volume";                                      break;
	case 132: error_string="market is closed";                                          break;
	case 133: error_string="trade is disabled";                                         break;
	case 134: error_string="not enough money";                                          break;
	case 135: error_string="price changed";                                             break;
	case 136: error_string="off quotes";                                                break;
	case 137: error_string="broker is busy";                                            break;
	case 138: error_string="requote";                                                   break;
	case 139: error_string="order is locked";                                           break;
	case 140: error_string="long positions only allowed";                               break;
	case 141: error_string="too many requests";                                         break;
	case 145: error_string="modification denied because order too close to market";     break;
	case 146: error_string="trade context is busy";                                     break;
		//---- mql4 errors
	case 4000: error_string="no error";                                                 break;
	case 4001: error_string="wrong function pointer";                                   break;
	case 4002: error_string="array index is out of range";                              break;
	case 4003: error_string="no memory for function call stack";                        break;
	case 4004: error_string="recursive stack overflow";                                 break;
	case 4005: error_string="not enough stack for parameter";                           break;
	case 4006: error_string="no memory for parameter string";                           break;
	case 4007: error_string="no memory for temp string";                                break;
	case 4008: error_string="not initialized string";                                   break;
	case 4009: error_string="not initialized string in array";                          break;
	case 4010: error_string="no memory for array\' string";                             break;
	case 4011: error_string="too long string";                                          break;
	case 4012: error_string="remainder from zero divide";                               break;
	case 4013: error_string="zero divide";                                              break;
	case 4014: error_string="unknown command";                                          break;
	case 4015: error_string="wrong jump (never generated error)";                       break;
	case 4016: error_string="not initialized array";                                    break;
	case 4017: error_string="dll calls are not allowed";                                break;
	case 4018: error_string="cannot load library";                                      break;
	case 4019: error_string="cannot call function";                                     break;
	case 4020: error_string="expert function calls are not allowed";                    break;
	case 4021: error_string="not enough memory for temp string returned from function"; break;
	case 4022: error_string="system is busy (never generated error)";                   break;
	case 4050: error_string="invalid function parameters count";                        break;
	case 4051: error_string="invalid function parameter value";                         break;
	case 4052: error_string="string function internal error";                           break;
	case 4053: error_string="some array error";                                         break;
	case 4054: error_string="incorrect series array using";                             break;
	case 4055: error_string="custom indicator error";                                   break;
	case 4056: error_string="arrays are incompatible";                                  break;
	case 4057: error_string="global variables processing error";                        break;
	case 4058: error_string="global variable not found";                                break;
	case 4059: error_string="function is not allowed in testing mode";                  break;
	case 4060: error_string="function is not confirmed";                                break;
	case 4061: error_string="send mail error";                                          break;
	case 4062: error_string="string parameter expected";                                break;
	case 4063: error_string="integer parameter expected";                               break;
	case 4064: error_string="double parameter expected";                                break;
	case 4065: error_string="array as parameter expected";                              break;
	case 4066: error_string="requested history data in update state";                   break;
	case 4099: error_string="end of file";                                              break;
	case 4100: error_string="some file error";                                          break;
	case 4101: error_string="wrong file name";                                          break;
	case 4102: error_string="too many opened files";                                    break;
	case 4103: error_string="cannot open file";                                         break;
	case 4104: error_string="incompatible access to a file";                            break;
	case 4105: error_string="no order selected";                                        break;
	case 4106: error_string="unknown symbol";                                           break;
	case 4107: error_string="invalid price parameter for trade function";               break;
	case 4108: error_string="invalid ticket";                                           break;
	case 4109: error_string="trade is not allowed";                                     break;
	case 4110: error_string="longs are not allowed";                                    break;
	case 4111: error_string="shorts are not allowed";                                   break;
	case 4200: error_string="object is already exist";                                  break;
	case 4201: error_string="unknown object property";                                  break;
	case 4202: error_string="object is not exist";                                      break;
	case 4203: error_string="unknown object type";                                      break;
	case 4204: error_string="no object name";                                           break;
	case 4205: error_string="object coordinates error";                                 break;
	case 4206: error_string="no specified subwindow";                                   break;
	default:   error_string="unknown error";
	}
	//----
	return(error_string);
}  
//+------------------------------------------------------------------+